package statechart

import org.specs._
import scala.io.Source._
import statechart.StatechartParser._

/**
 * Tests for the statechart parser.
 */
object StatechartParserSpec extends Specification("statechart.StatechartParser") {

  def scan(s: String) = new lexical.Scanner(s)

  "StatechartParser.state" should {
  
    def parseState(s: String) = StatechartParser.state(scan(s))
  
    "match a good state" in {
      val s = """state DoorOpen:  
        onEntry: turnLightOn
        onExit: turnLightOff
        transitions:
          closeDoor -> Ready
      """
      val r = StatechartParser.state(scan(s))
      r must haveClass[Success[Any]]
    }
    
    "not match an incomplete state" in {
      val s = """state BadState"""
      val r = StatechartParser.state(scan(s))
      r must haveClass[Failure]
    }
    
    "match a state with two transitions" in {
      val r = parseState("""state TwoTransitions:
                              transitions:
                                action1 -> FirstNext
                                action2 -> SecondNext
                                """)
      r must haveClass[Success[Any]]
    }
    
    "match a state with comparison conditions" in {
      val s = """extendedState Ready:
                    substates: Idle, Cooking
                    initialState: Idle
                    transitions:    
                      start[cookingTime gt 0] -> Cooking
                      start[cookingTime eq 0] -> Idle
                      openDoor -> DoorOpen"""
      val r = StatechartParser.extendedState(scan(s))
      r must haveClass[Success[Any]]
    }
  }
  
  "StatechartParser.statechart" should {
    "match the microwave oven definition" in {
      val s = fromFile("microwave.txt").mkString    
      val r = StatechartParser.statechart(scan(s))
      r must haveClass[Success[Any]]
    }
  }
}
